#!/bin/sh -e

configVRouter()
{
	cat juju-header
	echo "auto $1"
	if [ -e "$2" ]; then
		cat "$2"
	else
		echo "iface $1 inet manual"
	fi
	printf "\n%s\n"	"auto vhost0"
	if [ -e "$3" ]; then
		cat "$3"
	else
		echo "iface vhost0 inet dhcp"
	fi
	cat <<-EOF
		    pre-up ip link add address \$(cat /sys/class/net/$1/address) type vhost
		    pre-up vif --add $1 --mac \$(cat /sys/class/net/$1/address) --vrf 0 --vhost-phys --type physical
		    pre-up vif --add vhost0 --mac \$(cat /sys/class/net/$1/address) --vrf 0 --type vhost --xconnect $1
		    post-down vif --list | awk '/^vif.*OS: vhost0/ {split(\$1, arr, "\\/"); print arr[2];}' | xargs vif --delete
		    post-down vif --list | awk '/^vif.*OS: $1/ {split(\$1, arr, "\\/"); print arr[2];}' | xargs vif --delete
		    post-down ip link delete vhost0
		EOF
}

ifacedown()
{
	for iface; do
		if [ ! -e /sys/class/net/$iface ]; then
			continue
		fi
		[ -d /sys/class/net/$iface/bridge ] && saveIfaces $iface
		ifdown $iface
	done
}

ifaceup()
{
	for iface; do
		ifup $iface
		[ -d /sys/class/net/$iface/bridge ] && restoreIfaces $iface
	done
	return 0
}

restoreIfaces()
{
	cat $TMP/$1.ifaces | xargs -n 1 brctl addif $1 || true
}

saveIfaces()
{
	find /sys/class/net/$1/brif | tail -n +2 | xargs -n 1 basename \
	    > $TMP/$1.ifaces
}

TMP=$(mktemp -d /tmp/create-vrouter.XXX)

if [ $# -ne 0 ]; then
	interface=$1
else
	interface=$(route -n | awk '$1 == "0.0.0.0" { print $8 }')
fi

ifacedown $interface vhost0; sleep 5
if ! grep -q '^[[:blank:]]*source /etc/network/interfaces\.d/\*\.cfg[[:blank:]]*$' \
    /etc/network/interfaces; then
	printf "\n%s\n" "source /etc/network/interfaces.d/*.cfg" \
	    >> /etc/network/interfaces
fi
mkdir -p /etc/network/interfaces.d
for cfg in /etc/network/interfaces /etc/network/interfaces.d/*.cfg \
    /etc/network/*.config; do
	[ -e "$cfg" ] || continue
	awk -v interface=$interface -v interface_cfg=$TMP/interface.cfg \
	    -v vrouter_cfg=$TMP/vrouter.cfg -f vrouter-interfaces.awk "$cfg" \
	    > $TMP/interfaces.cfg
	if ! diff $TMP/interfaces.cfg "$cfg" > /dev/null; then
		mv "$cfg" "$cfg.save"
		cat juju-header $TMP/interfaces.cfg > "$cfg"
	fi
done
configVRouter $interface $TMP/interface.cfg $TMP/vrouter.cfg \
    > /etc/network/interfaces.d/vrouter.cfg
ifaceup $interface vhost0

rm -rf $TMP
